import matplotlib.pyplot as plt

from outset.stub import stub_edge_mark


def test_stub_edge_mark_up():
    ax = plt.gca()  # Get the current axes=
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)

    # Call the function with test values
    stub_edge_mark(x=0.5, y=5, ax=ax)

    # Save and print the path of the output image
    outpath = "/tmp/test_stub_edge_mark_up.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")


def test_stub_edge_mark_down():
    ax = plt.gca()  # Get the current axes
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)

    # Call the function with test values
    stub_edge_mark(x=0.5, y=-5, ax=ax)

    # Save and print the path of the output image
    outpath = "/tmp/test_stub_edge_mark_down.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")


def test_stub_edge_mark_left():
    ax = plt.gca()  # Get the current axes
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)

    # Call the function with test values
    stub_edge_mark(x=5, y=0.85, ax=ax)

    # Save and print the path of the output image
    outpath = "/tmp/test_stub_edge_mark_left.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")


def test_stub_edge_mark_right():
    ax = plt.gca()  # Get the current axes
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)

    # Call the function with test values
    stub_edge_mark(x=-5, y=0.25, ax=ax)

    # Save and print the path of the output image
    outpath = "/tmp/test_stub_edge_mark_right.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")



import typing

import frozendict
from matplotlib import axes as mpl_axes
from matplotlib import patches as mpl_patches
from matplotlib import pyplot as plt


def draw_frame(
    frame_xlim: typing.Tuple[float, float],
    frame_ylim: typing.Tuple[float, float],
    ax: typing.Optional[mpl_axes.Axes] = None,
    frame_edge_kws: typing.Dict = frozendict.frozendict(),
    frame_face_kws: typing.Dict = frozendict.frozendict(),
    **kwargs,
) -> mpl_axes.Axes:
    """Mark a rectangular region with a frame border and an underlaid color
    underlay fill.

    Parameters
    ----------
    frame_xlim : Tuple[float, float]
        The x-limits of the rectangular frame in the form (xmin, xmax).
    frame_ylim : Tuple[float, float]
        The y-limits of the rectangular frame in the form (ymin, ymax).
    ax : matplotlib.axes.Axes, optional
        The axes object on which to draw. If None, the current active axes will
        be used.
    frame_edge_kws : Dict, default {}
        Keyword arguments for customizing the frame's edge appearance.
    frame_face_kws : Dict, default {}
        Keyword arguments for customizing the frame's face appearance --- i.e.,
        the underlaid solid fill.
    **kwargs
        Additional keyword arguments for the matplotlib `Rectangle` used to draw
        the frame.

    Returns
    -------
    mpl_axes.Axes
        The modified matplotlib axes object with the frame drawing elements added.
    """
    if ax is None:
        ax = plt.gca()

    frame_face_patch = mpl_patches.Rectangle(
        (frame_xlim[0], frame_ylim[0]),  # lower left corner
        frame_xlim[1] - frame_xlim[0],  # width
        frame_ylim[1] - frame_ylim[0],  # height
        **{
            "alpha": 0.1,
            **kwargs,
            "edgecolor": "none",
            "zorder": -1,
            **frame_face_kws,
        },
    )
    ax.add_patch(frame_face_patch)

    frame_edge_patch = mpl_patches.Rectangle(
        (frame_xlim[0], frame_ylim[0]),  # lower left corner
        frame_xlim[1] - frame_xlim[0],  # width
        frame_ylim[1] - frame_ylim[0],  # height
        **{
            **kwargs,
            "facecolor": "none",
            **frame_edge_kws,
        },
    )
    ax.add_patch(frame_edge_patch)

    return ax



from outset.util import NamedFrames


def test_NamedFrame_kwarg_initialization():
    assert NamedFrames(foo=(1, 2, 3, 4))["foo"] == (1, 2, 3, 4)


def test_NamedFrame_dict_initialization():
    assert NamedFrames({"foo": (1, 2, 3, 4)})["foo"] == (1, 2, 3, 4)



import typing
import matplotlib.pyplot as plt
from matplotlib import axes as mpl_axes
import numpy as np


class TweakReflect:
    """Functor to change orientation of callout via reflection.

    The adjustment is applied by reflecting the leader vertices about the
    centroid of the frame. Horizontal and/or vertical reflection can be
    specified.
    """

    _horizontal: bool
    _vertical: bool

    def __init__(
        self: "TweakReflect",
        *,
        horizontal: typing.Optional[bool] = None,
        vertical: typing.Optional[bool] = None,
    ) -> None:
        """
        Initialize the TweakReflect functor.

        If neither `horizontal` or `vertical` are specified, a horizontal
        reflection will be performed.

        Parameters
        ----------
        horizontal : Optional[bool]
            Should the callout leader be reflected horizontally?
        vertical : Optional[Tuple[float, float]], optional
            Should the callout leader be reflected vertically?
        """
        if horizontal is None and vertical is None:
            self._horizontal = True
            self._vertical = False
        elif horizontal is None:
            self._horizontal = False
            self._vertical = vertical
        elif vertical is None:
            self._horizontal = horizontal
            self._vertical = False
        else:
            self._horizontal = horizontal
            self._vertical = vertical

    def __call__(
        self: "TweakReflect",
        leader_vertices: typing.Sequence[typing.Tuple[float, float]],
        ax: typing.Optional[mpl_axes.Axes] = None,
    ) -> typing.List[typing.Tuple[float, float]]:
        """Reflect leader vertices about centroid.

        Parameters
        ----------
        leader_vertices : Sequence[Tuple[float, float]]
            A sequence of tuples representing the `(x, y)` coordinates of the
            leader vertices.
        ax : mpl_axes.Axes
            The matplotlib Axes object on which the leader vertices are plotted.

            Not used for this tweak.
        Returns
        -------
        List[Tuple[float, float]]
            The updated leader vertices.
        """
        v0, __, v1, __ = leader_vertices

        cx, cy = np.mean([v0, v1], axis=0)

        if self._horizontal:
            leader_vertices = [(2 * cx - x, y) for x, y in leader_vertices]
        if self._vertical:
            leader_vertices = [(x, 2 * cy - y) for x, y in leader_vertices]

        return leader_vertices



"""Markers for marquee annotations."""

from ._MarkArrow import mark_arrow, MarkArrow
from ._MarkInlaidAsterisk import mark_inlaid_asterisk, MarkInlaidAsterisk
from ._MarkMagnifyingGlass import mark_magnifying_glass, MarkMagnifyingGlass
from ._MarkAlphabeticalBadges import MarkAlphabeticalBadges
from ._MarkNumericalBadges import MarkNumericalBadges
from ._MarkRomanBadges import MarkRomanBadges

__all__ = [
    "mark_arrow",
    "mark_inlaid_asterisk",
    "mark_magnifying_glass",
    "MarkAlphabeticalBadges",
    "MarkArrow",
    "MarkInlaidAsterisk",
    "MarkMagnifyingGlass",
    "MarkNumericalBadges",
    "MarkRomanBadges",
]



import matplotlib.pyplot as plt

from outset.mark._MarkNumericalBadges import MarkNumericalBadges


def test_MarkNumericalBadges():
    ax = plt.gca()  # Get the current axes

    ftor = MarkNumericalBadges()
    # Call the function with test values
    ftor(x=0.5, y=0.5, ax=ax, color="red")
    ftor(
        x=1.5,
        y=1.5,
        ax=ax,
        color="red",
        scale_numeral=0.6,
    )

    # Save and print the path of the output image
    outpath = "/tmp/test_MarkNumericalBadges.png"
    plt.savefig(outpath)
    print(f"saved graphic to {outpath}")
